"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const types_1 = require("./types");
const convert_1 = require("../convert");
const errors_1 = require("../errors");
const metadata_registry_1 = require("../metadata-registry");
const diagnosticUtil_1 = require("./diagnosticUtil");
const fast_xml_parser_1 = require("fast-xml-parser");
const treeContainers_1 = require("../metadata-registry/treeContainers");
exports.DEFAULT_API_OPTIONS = {
    rollbackOnError: true,
    ignoreWarnings: false,
    checkOnly: false,
    singlePackage: true,
};
class MetadataApi extends types_1.BaseApi {
    deploy(components, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const metadataComponents = Array.isArray(components) ? components : [components];
            const converter = new convert_1.MetadataConverter();
            const { zipBuffer } = yield converter.convert(metadataComponents, 'metadata', { type: 'zip' });
            const deployID = yield this.metadataDeployID(zipBuffer, options);
            const deployStatusPoll = this.metadataDeployStatusPoll(deployID, options);
            const componentDeploymentMap = new Map();
            for (const component of metadataComponents) {
                componentDeploymentMap.set(`${component.type.name}:${component.fullName}`, {
                    status: types_1.ComponentStatus.Unchanged,
                    component,
                    diagnostics: [],
                });
            }
            const result = yield deployStatusPoll;
            return this.buildSourceDeployResult(componentDeploymentMap, result);
        });
    }
    deployWithPaths(paths, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const components = this.resolver.getComponentsFromPath(paths);
            return this.deploy(components, options);
        });
    }
    // TODO: W-8023153: move filtering logic to registry
    retrieveWithPaths(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const allComponents = [];
            for (const filepath of options.paths) {
                allComponents.push(...this.resolver.getComponentsFromPath(filepath));
            }
            const hashedCmps = new Set();
            const uniqueComponents = allComponents.filter((component) => {
                const hashed = this.hashElement(component);
                if (!hashedCmps.has(hashed)) {
                    hashedCmps.add(hashed);
                    return component;
                }
            });
            const retrieveOptions = { components: uniqueComponents };
            return this.retrieve(Object.assign(retrieveOptions, options));
        });
    }
    retrieve(options) {
        return __awaiter(this, void 0, void 0, function* () {
            let components = [];
            const retrieveRequest = this.formatRetrieveRequest(options.components);
            const retrieveResult = yield this.getRetrievedResult(retrieveRequest, options);
            if (retrieveResult.status === types_1.RetrieveStatus.Succeeded) {
                const tree = yield treeContainers_1.ZipTreeContainer.create(Buffer.from(retrieveResult.zipFile, 'base64'));
                const zipComponents = new metadata_registry_1.MetadataResolver(undefined, tree).getComponentsFromPath('.');
                components = yield this.getConvertedComponents(zipComponents, options);
            }
            const componentRetrievals = components.map((component) => {
                return { component, status: retrieveResult.status };
            });
            const sourceRetrieveResult = this.buildSourceRetrieveResult(retrieveResult, options, componentRetrievals);
            return sourceRetrieveResult;
        });
    }
    formatRetrieveRequest(components) {
        const manifestGenerator = new metadata_registry_1.ManifestGenerator(this.resolver, this.registry);
        const manifest = manifestGenerator.createManifest(components);
        const manifestJson = fast_xml_parser_1.parse(manifest);
        const packageData = manifestJson.Package;
        delete packageData.$;
        const retrieveRequest = {
            apiVersion: this.registry.apiVersion,
            unpackaged: packageData,
        };
        return retrieveRequest;
    }
    getRetrievedResult(retrieveRequest, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // @ts-ignore required callback
            const retrieveId = (yield this.connection.metadata.retrieve(retrieveRequest)).id;
            const retrieveResult = yield this.metadataRetrieveStatusPoll(retrieveId, options);
            return retrieveResult;
        });
    }
    metadataRetrieveStatusPoll(retrieveID, options, interval = 100) {
        return __awaiter(this, void 0, void 0, function* () {
            let result;
            const wait = (interval) => {
                return new Promise((resolve) => {
                    setTimeout(resolve, interval);
                });
            };
            const timeout = !options || !options.wait ? 10000 : options.wait;
            const endTime = Date.now() + timeout;
            let triedOnce = false;
            do {
                if (triedOnce) {
                    yield wait(interval);
                }
                try {
                    // Recasting to use the library's RetrieveResult type
                    result = (yield this.connection.metadata.checkRetrieveStatus(retrieveID));
                }
                catch (e) {
                    throw new errors_1.RetrieveError('md_request_fail', e);
                }
                switch (result.status) {
                    case types_1.RetrieveStatus.Succeeded:
                    case types_1.RetrieveStatus.Failed:
                        return result;
                    case types_1.RetrieveStatus.InProgress:
                }
                triedOnce = true;
            } while (Date.now() < endTime);
            return result;
        });
    }
    buildSourceRetrieveResult(retrieveResult, options, components) {
        const success = this.calculateSuccess(retrieveResult, options, components);
        const sourceRetrieveResult = {
            status: success ? types_1.RetrieveStatus.Succeeded : types_1.RetrieveStatus.Failed,
            id: retrieveResult.id,
            success,
        };
        sourceRetrieveResult.components = components || [];
        sourceRetrieveResult.messages = [];
        if (retrieveResult.hasOwnProperty('messages')) {
            const diagnosticUtil = new diagnosticUtil_1.DiagnosticUtil('metadata');
            const messages = Array.isArray(retrieveResult.messages)
                ? retrieveResult.messages
                : [retrieveResult.messages];
            for (const retrieveMessage of messages) {
                let existingRetrieval;
                let failedComponent;
                const matches = retrieveMessage.problem.match(/.+'(.+)'.+'(.+)'/);
                if (matches && Array.isArray(matches)) {
                    const [fullName] = matches.slice(2);
                    existingRetrieval = components.find((retrieval) => {
                        return retrieval.component.fullName === fullName;
                    });
                    failedComponent = options.components.find((component) => {
                        return component.fullName === fullName;
                    });
                }
                if (existingRetrieval) {
                    diagnosticUtil.setRetrieveDiagnostic(retrieveMessage.problem, existingRetrieval);
                }
                else if (failedComponent) {
                    const failedRetrieval = {
                        component: failedComponent,
                        status: sourceRetrieveResult.status,
                    };
                    diagnosticUtil.setRetrieveDiagnostic(retrieveMessage.problem, failedRetrieval);
                    sourceRetrieveResult.components.push(failedRetrieval);
                }
                else {
                    sourceRetrieveResult.messages.push(retrieveMessage);
                }
            }
        }
        return sourceRetrieveResult;
    }
    calculateSuccess(retrieveResult, options, components) {
        return ((retrieveResult.status === types_1.RetrieveStatus.Succeeded &&
            options.components.length === components.length &&
            !retrieveResult.hasOwnProperty('messages')) ||
            retrieveResult.status === types_1.RetrieveStatus.InProgress ||
            retrieveResult.status === types_1.RetrieveStatus.Pending);
    }
    getConvertedComponents(retrievedComponents, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const converter = new convert_1.MetadataConverter();
            const outputConfig = options.merge
                ? {
                    type: 'merge',
                    mergeWith: options.components,
                    defaultDirectory: options.output,
                }
                : {
                    type: 'directory',
                    outputDirectory: options.output,
                };
            const convertResult = yield converter.convert(retrievedComponents, 'source', outputConfig);
            if (options.merge) {
                // TODO: W-8220616: this may return incomplete information about the retrieve
                return options.components;
            }
            return this.resolver.getComponentsFromPath(convertResult.packagePath);
        });
    }
    hashElement(component) {
        const hashed = `${component.fullName}.${component.type.id}`;
        return hashed;
    }
    metadataDeployID(zipBuffer, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!options || !options.apiOptions) {
                options = {
                    apiOptions: exports.DEFAULT_API_OPTIONS,
                };
            }
            else {
                for (const [property, value] of Object.entries(exports.DEFAULT_API_OPTIONS)) {
                    if (!(property in options.apiOptions)) {
                        //@ts-ignore ignore while dynamically building the defaults
                        options.apiOptions[property] = value;
                    }
                }
            }
            const result = yield this.connection.metadata.deploy(zipBuffer, options.apiOptions);
            return result.id;
        });
    }
    metadataDeployStatusPoll(deployID, options, interval = 100) {
        return __awaiter(this, void 0, void 0, function* () {
            let result;
            const wait = (interval) => {
                return new Promise((resolve) => {
                    setTimeout(resolve, interval);
                });
            };
            const timeout = !options || !options.wait ? 10000 : options.wait;
            const endTime = Date.now() + timeout;
            let triedOnce = false;
            do {
                if (triedOnce) {
                    yield wait(interval);
                }
                try {
                    // Recasting to use the library's DeployResult type
                    result = (yield this.connection.metadata.checkDeployStatus(deployID, true));
                }
                catch (e) {
                    throw new errors_1.DeployError('md_request_fail', e);
                }
                switch (result.status) {
                    case types_1.DeployStatus.Succeeded:
                    case types_1.DeployStatus.Failed:
                    case types_1.DeployStatus.Canceled:
                        return result;
                }
                triedOnce = true;
            } while (Date.now() < endTime);
            return result;
        });
    }
    buildSourceDeployResult(componentDeploymentMap, result) {
        const deployResult = {
            id: result.id,
            status: result.status,
            success: result.success,
        };
        const messages = this.getDeployMessages(result);
        const diagnosticUtil = new diagnosticUtil_1.DiagnosticUtil('metadata');
        if (messages.length > 0) {
            deployResult.components = [];
            for (let message of messages) {
                message = this.sanitizeDeployMessage(message);
                const componentKey = `${message.componentType}:${message.fullName}`;
                const componentDeployment = componentDeploymentMap.get(componentKey);
                if (componentDeployment) {
                    if (message.created === 'true') {
                        componentDeployment.status = types_1.ComponentStatus.Created;
                    }
                    else if (message.changed === 'true') {
                        componentDeployment.status = types_1.ComponentStatus.Changed;
                    }
                    else if (message.deleted === 'true') {
                        componentDeployment.status = types_1.ComponentStatus.Deleted;
                    }
                    else if (message.success === 'false') {
                        componentDeployment.status = types_1.ComponentStatus.Failed;
                    }
                    if (message.problem) {
                        diagnosticUtil.setDeployDiagnostic(componentDeployment, message);
                    }
                }
            }
            deployResult.components = Array.from(componentDeploymentMap.values());
        }
        return deployResult;
    }
    getDeployMessages(result) {
        const messages = [];
        if (result.details) {
            const { componentSuccesses, componentFailures } = result.details;
            if (componentSuccesses) {
                if (Array.isArray(componentSuccesses)) {
                    messages.push(...componentSuccesses);
                }
                else {
                    messages.push(componentSuccesses);
                }
            }
            if (componentFailures) {
                if (Array.isArray(componentFailures)) {
                    messages.push(...componentFailures);
                }
                else {
                    messages.push(componentFailures);
                }
            }
        }
        return messages;
    }
    /**
     * Fix any issues with the deploy message returned by the api.
     * TODO: remove as fixes are made in the api.
     */
    sanitizeDeployMessage(message) {
        // lwc doesn't properly use the fullname property in the api.
        message.fullName = message.fullName.replace(/markup:\/\/c:/, '');
        return message;
    }
}
exports.MetadataApi = MetadataApi;
//# sourceMappingURL=metadataApi.js.map